-- # Script Cinemythics # --
-- # 1.0.0 # --

SET SQL DIALECT 3;



/******************************************************************************/
/*                                   Tables                                   */
/******************************************************************************/


CREATE GENERATOR VERSION_ID;

CREATE TABLE VERSION_BASE (
    ID_MODIF      INTEGER NOT NULL,
    NUM_VERSION     VARCHAR(50) NOT NULL,
    SCRIPT_MODIF  VARCHAR(32000)
);


CREATE TABLE PSEUDOS (
    NUMGENS1  INTEGER NOT NULL,
    NUMGENS2  INTEGER NOT NULL
);


CREATE TABLE XTRA_GENS (
    NUMXTRA   INTEGER NOT NULL,
    NUMGENS   INTEGER NOT NULL,
    TYPEXTRA  VARCHAR(15)
);




/******************************************************************************/
/*                                Primary Keys                                */
/******************************************************************************/

ALTER TABLE VERSION_BASE ADD CONSTRAINT PK_VERSION_BASE PRIMARY KEY (ID_MODIF)
USING INDEX IDX_PK_VERSION_BASE;


ALTER TABLE PSEUDOS ADD CONSTRAINT IDX_PK_PSEUDOS PRIMARY KEY (NUMGENS1, NUMGENS2);


ALTER TABLE XTRA_GENS ADD CONSTRAINT PK_XTRA_GENS PRIMARY KEY (NUMXTRA, NUMGENS)
USING INDEX IDX_PK_XTRA_GENS;


/******************************************************************************/
/*                                Foreign Keys                                */
/******************************************************************************/

ALTER TABLE PSEUDOS ADD CONSTRAINT FK_PSEUDOS_GENS_NUMGENS1 FOREIGN KEY (NUMGENS1) REFERENCES GENS (NUMGENS);
ALTER TABLE PSEUDOS ADD CONSTRAINT FK_PSEUDOS_GENS_NUMGENS2 FOREIGN KEY (NUMGENS2) REFERENCES GENS (NUMGENS);


ALTER TABLE XTRA_GENS ADD CONSTRAINT FK_XTRA_GENS_GENS FOREIGN KEY (NUMGENS) REFERENCES GENS (NUMGENS)
  USING INDEX IDX_FK_XTRA_GENS_GENS;
ALTER TABLE XTRA_GENS ADD CONSTRAINT FK_XTRA_GENS_MTD FOREIGN KEY (NUMXTRA) REFERENCES MULTIMEDIA (NUMXTRA)
  USING INDEX IDX_FK_XTRA_GENS_MTD;


/******************************************************************************/
/*                            Triggers for tables                             */
/******************************************************************************/



/* Trigger: TR_VERSION_BASE_BI */
CREATE OR ALTER TRIGGER TR_VERSION_BASE_BI FOR VERSION_BASE
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
/* Trigger text */
if (New.ID_MODIF is NULL) then
NEW.ID_MODIF = gen_id(version_id, 1);
END
;



/* Trigger: TR_PSEUDOS_BI */
CREATE OR ALTER TRIGGER TR_PSEUDOS_BI FOR PSEUDOS
ACTIVE BEFORE INSERT POSITION 0
AS
declare
    tmpnumgens integer;
begin
  /* Trigger text */
  if (new.numgens1 > new.numgens2) then
   begin
     tmpnumgens = new.numgens1;
     new.numgens1 = new.numgens2;
     new.numgens2 = tmpnumgens;
   end
end
;



CREATE OR ALTER TRIGGER TR_MULTIMEDIA_BD FOR MULTIMEDIA
ACTIVE BEFORE DELETE POSITION 0
AS
begin
  /* Trigger text */
  /* Table Xtra_MVI */
   delete from xtra_mvi where NumXtra = old.numxtra ;
  /* Table Xtra_GENS */
   delete from xtra_gens where NumXtra = old.numxtra ;
end;




CREATE OR ALTER TRIGGER TR_GENS_BD FOR GENS
ACTIVE BEFORE DELETE POSITION 0
AS
begin
  /* Trigger text */
  /*--Table Tag_Gens  */
  Delete from tag_gens where NumGens = old.numgens ;
Delete from pseudos where NumGens1 = old.numgens;
Delete from pseudos where NumGens2 = old.numgens;
end;









/******************************************************************************/
/*                                 Privileges                                 */
/******************************************************************************/


/* Privileges of roles */
GRANT ALL ON VERSION_BASE TO ADMINCINE;



/* Privileges of roles */
GRANT ALL ON PSEUDOS TO ADMINCINE;
GRANT ALL ON XTRA_GENS TO ADMINCINE;


/* Privileges of users */
GRANT ALL ON XTRA_GENS TO CINE2000 WITH GRANT OPTION;
GRANT ALL ON PSEUDOS TO CINE2000 WITH GRANT OPTION;


-- ---------------------------------------------------------------------------

-- Insertion des valeurs
insert into VERSION_BASE(NUM_VERSION, SCRIPT_MODIF) values (
'1.0.1',
'
-- # Script Cinemythics # --
-- # 1.0.0 # --

SET SQL DIALECT 3;


/******************************************************************************/
/*                                   Tables                                   */
/******************************************************************************/


CREATE GENERATOR VERSION_ID;

CREATE TABLE VERSION_BASE (
    ID_MODIF      INTEGER NOT NULL,
    NUM_VERSION     VARCHAR(50) NOT NULL,
    SCRIPT_MODIF  VARCHAR(32000)
);


CREATE TABLE PSEUDOS (
    NUMGENS1  INTEGER NOT NULL,
    NUMGENS2  INTEGER NOT NULL
);


CREATE TABLE XTRA_GENS (
    NUMXTRA   INTEGER NOT NULL,
    NUMGENS   INTEGER NOT NULL,
    TYPEXTRA  VARCHAR(15)
);




/******************************************************************************/
/*                                Primary Keys                                */
/******************************************************************************/

ALTER TABLE VERSION_BASE ADD CONSTRAINT PK_VERSION_BASE PRIMARY KEY (ID_MODIF)
USING INDEX IDX_PK_VERSION_BASE;


ALTER TABLE PSEUDOS ADD CONSTRAINT IDX_PK_PSEUDOS PRIMARY KEY (NUMGENS1, NUMGENS2);


ALTER TABLE XTRA_GENS ADD CONSTRAINT PK_XTRA_GENS PRIMARY KEY (NUMXTRA, NUMGENS)
USING INDEX IDX_PK_XTRA_GENS;


/******************************************************************************/
/*                                Foreign Keys                                */
/******************************************************************************/

ALTER TABLE PSEUDOS ADD CONSTRAINT FK_PSEUDOS_GENS_NUMGENS1 FOREIGN KEY (NUMGENS1) REFERENCES GENS (NUMGENS);
ALTER TABLE PSEUDOS ADD CONSTRAINT FK_PSEUDOS_GENS_NUMGENS2 FOREIGN KEY (NUMGENS2) REFERENCES GENS (NUMGENS);


ALTER TABLE XTRA_GENS ADD CONSTRAINT FK_XTRA_GENS_GENS FOREIGN KEY (NUMGENS) REFERENCES GENS (NUMGENS)
  USING INDEX IDX_FK_XTRA_GENS_GENS;
ALTER TABLE XTRA_GENS ADD CONSTRAINT FK_XTRA_GENS_MTD FOREIGN KEY (NUMXTRA) REFERENCES MULTIMEDIA (NUMXTRA)
  USING INDEX IDX_FK_XTRA_GENS_MTD;


/******************************************************************************/
/*                            Triggers for tables                             */
/******************************************************************************/



/* Trigger: TR_VERSION_BASE_BI */
CREATE OR ALTER TRIGGER TR_VERSION_BASE_BI FOR VERSION_BASE
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
/* Trigger text */
if (New.ID_MODIF is NULL) then
NEW.ID_MODIF = gen_id(version_id, 1);
END
;



/* Trigger: TR_PSEUDOS_BI */
CREATE OR ALTER TRIGGER TR_PSEUDOS_BI FOR PSEUDOS
ACTIVE BEFORE INSERT POSITION 0
AS
declare
    tmpnumgens integer;
begin
  /* Trigger text */
  if (new.numgens1 > new.numgens2) then
   begin
     tmpnumgens = new.numgens1;
     new.numgens1 = new.numgens2;
     new.numgens2 = tmpnumgens;
   end
end
;



CREATE OR ALTER TRIGGER TR_MULTIMEDIA_BD FOR MULTIMEDIA
ACTIVE BEFORE DELETE POSITION 0
AS
begin
  /* Trigger text */
  /* Table Xtra_MVI */
   delete from xtra_mvi where NumXtra = old.numxtra ;
  /* Table Xtra_GENS */
   delete from xtra_gens where NumXtra = old.numxtra ;
end;




CREATE OR ALTER TRIGGER TR_GENS_BD FOR GENS
ACTIVE BEFORE DELETE POSITION 0
AS
begin
  /* Trigger text */
  /*--Table Tag_Gens  */
  Delete from tag_gens where NumGens = old.numgens ;
Delete from pseudos where NumGens1 = old.numgens;
Delete from pseudos where NumGens2 = old.numgens;
end;









/******************************************************************************/
/*                                 Privileges                                 */
/******************************************************************************/


/* Privileges of roles */
GRANT ALL ON VERSION_BASE TO ADMINCINE;



/* Privileges of roles */
GRANT ALL ON PSEUDOS TO ADMINCINE;
GRANT ALL ON XTRA_GENS TO ADMINCINE;


/* Privileges of users */
GRANT ALL ON XTRA_GENS TO CINE2000 WITH GRANT OPTION;
GRANT ALL ON PSEUDOS TO CINE2000 WITH GRANT OPTION;'
);